@@ -222,8 +222,8 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
      * until the timeout period expires.
      */
     else if (errorCode == CM_ERROR_NOSUCHVOLUME) {
+	osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_NOSUCHVOLUME.");
         if (timeLeft > 7) {
-            osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_NOSUCHVOLUME.");
             thrd_Sleep(5000);
             
             retry = 1;
@@ -233,47 +233,57 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
         }
     }
 
+    else if (errorCode == CM_ERROR_ALLDOWN) {
+	osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_ALLDOWN.");
+	/* Servers marked DOWN will be restored by the background daemon
+	 * thread as they become available.
+	 */
+    }
+
     else if (errorCode == CM_ERROR_ALLOFFLINE) {
         if (timeLeft > 7) {
             osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_ALLOFFLINE.");
             thrd_Sleep(5000);
             
-            /* cm_ForceUpdateVolume marks all servers as non_busy */
-            /* No it doesn't and it won't do anything if all of the 
-             * the servers are marked as DOWN.  So clear the DOWN
-             * flag and reset the busy state as well.
-             */
-            if (!serversp) {
-                code = cm_GetServerList(fidp, userp, reqp, &serverspp);
-                if (code == 0) {
-                    serversp = *serverspp;
-                    free_svr_list = 1;
-                }
-            }
-            if (serversp) {
-                lock_ObtainWrite(&cm_serverLock);
-                for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
-                    tsrp->server->flags &= ~CM_SERVERFLAG_DOWN;
-                    if (tsrp->status == busy)
-                        tsrp->status = not_busy;
-                }
-                lock_ReleaseWrite(&cm_serverLock);
-                if (free_svr_list) {
-                    cm_FreeServerList(&serversp);
-                    *serverspp = serversp;
-                }
-                retry = 1;
-            }
+	    if (fidp) {	/* Not a VLDB call */
+		if (!serversp) {
+		    code = cm_GetServerList(fidp, userp, reqp, &serverspp);
+		    if (code == 0) {
+			serversp = *serverspp;
+			free_svr_list = 1;
+		    }
+		}
+		if (serversp) {
+		    lock_ObtainWrite(&cm_serverLock);
+		    for (tsrp = serversp; tsrp; tsrp=tsrp->next)
+			tsrp->status = not_busy;
+		    lock_ReleaseWrite(&cm_serverLock);
+		    if (free_svr_list) {
+			cm_FreeServerList(&serversp);
+			*serverspp = serversp;
+		    }
+		    retry = 1;
+		}
 
-            if (fidp != NULL)   /* Not a VLDB call */
                 cm_ForceUpdateVolume(fidp, userp, reqp);
-			else
-				retry = 0;
+	    } else { /* VLDB call */
+		if (serversp) {
+		    lock_ObtainWrite(&cm_serverLock);
+		    for (tsrp = serversp; tsrp; tsrp=tsrp->next)
+			tsrp->status = not_busy;
+		    lock_ReleaseWrite(&cm_serverLock);
+		    if (free_svr_list) {
+			cm_FreeServerList(&serversp);
+			*serverspp = serversp;
+		    }
+		}
+	    }	
         }
     }
 
     /* if all servers are busy, mark them non-busy and start over */
     else if (errorCode == CM_ERROR_ALLBUSY) {
+	osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_ALLBUSY.");
         if (timeLeft > 7) {
             thrd_Sleep(5000);
             if (!serversp) {
@@ -326,30 +336,53 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     else if (errorCode == VNOVOL || errorCode == VMOVED || errorCode == VOFFLINE ||
              errorCode == VSALVAGE || errorCode == VNOSERVICE || errorCode == VIO) 
     {       
-        /* Log server being offline for this volume */
-        osi_Log4(afsd_logp, "cm_Analyze found server %d.%d.%d.%d marked offline for a volume",
-                  ((serverp->addr.sin_addr.s_addr & 0xff)),
-                  ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
-                  ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
-                  ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24));
-        /* Create Event Log message */ 
-        {
-            HANDLE h;
-            char *ptbuf[1];
-            char s[100];
-            h = RegisterEventSource(NULL, AFS_DAEMON_EVENT_NAME);
-            sprintf(s, "cm_Analyze: Server %d.%d.%d.%d reported volume %d as missing.",
-                     ((serverp->addr.sin_addr.s_addr & 0xff)),
-                     ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
-                     ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
-                     ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24),
-                     fidp->volume);
-            ptbuf[0] = s;
-            ReportEvent(h, EVENTLOG_WARNING_TYPE, 0, 1009, NULL,
-                         1, 0, ptbuf, NULL);
-            DeregisterEventSource(h);
+#ifndef DJGPP
+        HANDLE h;
+        char *ptbuf[1];
+        char s[100];
+#endif
+        char addr[16];
+        char *format;
+
+        switch ( errorCode ) {
+        case VNOVOL:
+            format = "Server %s reported volume %d as not attached.";
+            break;
+        case VMOVED:
+            format = "Server %s reported volume %d as moved.";
+            break;
+        case VOFFLINE:
+            format = "Server %s reported volume %d as offline.";
+            break;
+        case VSALVAGE:
+            format = "Server %s reported volume %d as needs salvage.";
+            break;
+        case VNOSERVICE:
+            format = "Server %s reported volume %d as not in service.";
+            break;
+        case VIO:
+            format = "Server %s reported volume %d as temporarily unaccessible.";
+            break;
         }
 
+        /* Log server being offline for this volume */
+        sprintf(addr, "%d.%d.%d.%d", 
+                 ((serverp->addr.sin_addr.s_addr & 0xff)),
+                 ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
+                 ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
+                 ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24)); 
+        osi_Log2(afsd_logp, format, osi_LogSaveString(afsd_logp,addr), fidp->volume);
+        
+#ifndef DJGPP
+        /* Create Event Log message */
+        h = RegisterEventSource(NULL, AFS_DAEMON_EVENT_NAME);
+        sprintf(s, format, addr, fidp->volume);
+        ptbuf[0] = s;
+        ReportEvent(h, EVENTLOG_WARNING_TYPE, 0, 1009, NULL,
+                     1, 0, ptbuf, NULL);
+        DeregisterEventSource(h);
+#endif
+
         /* Mark server offline for this volume */
         if (!serversp) {
             code = cm_GetServerList(fidp, userp, reqp, &serverspp);
@@ -473,12 +506,17 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
     cm_serverRef_t *tsrp;
     cm_server_t *tsp;
     long firstError = 0;
-    int someBusy = 0, someOffline = 0, allBusy = 1, allDown = 1;
+    int someBusy = 0, someOffline = 0, allOffline = 1, allBusy = 1, allDown = 1;
     long timeUsed, timeLeft, hardTimeLeft;
 #ifdef DJGPP
     struct timeval now;
 #endif /* DJGPP */        
 
+    if (serversp == NULL) {
+	osi_Log1(afsd_logp, "cm_ConnByMServers returning 0x%x", CM_ERROR_NOSUCHVOLUME);
+	return CM_ERROR_NOSUCHVOLUME;
+    }
+
     *connpp = NULL;
 
 #ifndef DJGPP
@@ -498,13 +536,15 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
         cm_GetServerNoLock(tsp);
         lock_ReleaseWrite(&cm_serverLock);
         if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
+	    allDown = 0;
             if (tsrp->status == busy) {
-                allDown = 0;
+		allOffline = 0;
                 someBusy = 1;
             } else if (tsrp->status == offline) {
-                someOffline = 1;
+		allBusy = 0;
+		someOffline = 1;
             } else {
-                allDown = 0;
+		allOffline = 0;
                 allBusy = 0;
                 code = cm_ConnByServer(tsp, usersp, connpp);
                 if (code == 0) {        /* cm_CBS only returns 0 */
@@ -531,15 +571,15 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
         lock_ObtainWrite(&cm_serverLock);
         cm_PutServerNoLock(tsp);
     }   
-
     lock_ReleaseWrite(&cm_serverLock);
+
     if (firstError == 0) {
-        if (serversp == NULL)
-            firstError = CM_ERROR_NOSUCHVOLUME;
-        else if (allDown) 
-            firstError = CM_ERROR_ALLOFFLINE;
+        if (allDown) 
+            firstError = CM_ERROR_ALLDOWN;
         else if (allBusy) 
             firstError = CM_ERROR_ALLBUSY;
+	else if (allOffline || (someBusy && someOffline))
+	    firstError = CM_ERROR_ALLOFFLINE;
         else {
             osi_Log0(afsd_logp, "cm_ConnByMServers returning impossible error TIMEDOUT");
             firstError = CM_ERROR_TIMEDOUT;
