@@ -32,14 +32,18 @@ extern afs_rwlock_t afs_xosi;
 
 /* osi_misc.c */
 extern int osi_lookupname(char *aname, uio_seg_t seg, int followlink,
-			  vnode_t ** dirvpp, struct dentry **dpp);
+			  struct dentry **dpp);
 extern int osi_InitCacheInfo(char *aname);
-extern int osi_rdwr(int rw, struct osi_file *file, caddr_t addrp,
-		    size_t asize, size_t * resid);
-extern int osi_file_uio_rdwr(struct osi_file *osifile, uio_t * uiop, int rw);
+extern int osi_rdwr(struct osi_file *osifile, uio_t * uiop, int rw);
 extern void afs_osi_SetTime(osi_timeval_t * tvp);
 extern void osi_linux_free_inode_pages(void);
-extern void check_bad_parent(struct dentry *dp);
+
+/* osi_probe.c */
+extern void *osi_find_syscall_table(int which);
+
+/* osi_syscall.c */
+extern int osi_syscall_init(void);
+extern void osi_syscall_clean(void);
 
 /* osi_sysctl.c */
 extern int osi_sysctl_init(void);
@@ -56,12 +60,11 @@ extern void osi_VM_Truncate(struct vcache *avc, int alen,
 			    struct AFS_UCRED *acred);
 
 /* osi_vfsops.c */
-extern void set_inode_cache(struct inode *ip, struct vattr *vp);
-extern void put_inode_on_dummy_list(struct inode *ip);
 extern void vattr2inode(struct inode *ip, struct vattr *vp);
+extern int afs_init_inodecache(void);
+extern void afs_destroy_inodecache(void);
 
 /* osi_vnodeops.c */
-extern int afs_linux_writepage_sync(struct inode *ip, struct page *pp,
-				    unsigned long offset, unsigned int count);
+extern void afs_fill_inode(struct inode *ip, struct vattr *vattr);
 
 #endif /* _OSI_PROTO_H_ */
